/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "SteppingSampleApl.h"


/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */
/* KEY ͏ */
typedef enum {
  STEPPING_KEY_IN_NONE,
  STEPPING_KEY_IN_A,
  STEPPING_KEY_IN_B,
  STEPPING_KEY_IN_C,
  STEPPING_KEY_IN_D
} STEPPING_KEY_IN_TYPE;

/* Stepping  */
typedef enum {
  STEPPING_STATE_A,
  STEPPING_STATE_AB,
  STEPPING_STATE_B,
  STEPPING_STATE_BC,
  STEPPING_STATE_C,
  STEPPING_STATE_CD,
  STEPPING_STATE_D,
  STEPPING_STATE_DA,
} STEPPING_STATE;

/* ^C}[ */
typedef enum {
  STEPPING_TIMER_STOP,
  STEPPING_TIMER_DOING
} STEPPING_TIMER_STATE;

/* ^C}[ݒl̑I */
typedef enum {
  STEPPING_SPEED_HIGH,
  STEPPING_SPEED_MEDIUM_HIGH,
  STEPPING_SPEED_MEDIUM,
  STEPPING_SPEED_MEDIUM_LOW,
  STEPPING_SPEED_LOW,
  STEPPING_SPEED_MAX
} STEPPING_SPEED;


/* PORT ݒ(b) */
#define STEPPING_A_PORT  (PD10)
#define STEPPING_B_PORT  (PD11)
#define STEPPING_C_PORT  (PD12)
#define STEPPING_D_PORT  (PD13)

/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static STEPPING_KEY_IN_TYPE stepping_key_in;
static STEPPING_STATE stepping_apl_state;
static STEPPING_TIMER_STATE stepping_timer_state;
static int stepping_timer_cnt;

static STEPPING_SPEED stepping_speed_state;
static int stepping_speed_value[STEPPING_SPEED_MAX] = {
  50, 100, 250, 500, 750
};

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static int stepping_apl_is_input (STEPPING_KEY_IN_TYPE *input_key);
static int stepping_apl_is_release (STEPPING_KEY_IN_TYPE input_key);
static void stepping_apl_start (STEPPING_STATE state);
static void stepping_apl_stop (void);
static void stepping_apl_set_next_state (STEPPING_KEY_IN_TYPE in_key);
static void stepping_apl_set_timer (void);
static void stepping_apl_set_timer_value (STEPPING_KEY_IN_TYPE in_key);

/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@SteppingAv̏                           */
/* ֐@F@SteppingApl_Init                                 */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
SteppingApl_Init (void)
{
  stepping_key_in      = STEPPING_KEY_IN_NONE;
  stepping_timer_state = STEPPING_TIMER_STOP;
  stepping_timer_cnt   = 0;
  stepping_apl_state   = STEPPING_STATE_A;
  stepping_speed_state = STEPPING_SPEED_MEDIUM;

  pio_direction (STEPPING_A_PORT, PIO_DIR_OUT);
  pio_direction (STEPPING_B_PORT, PIO_DIR_OUT);
  pio_direction (STEPPING_C_PORT, PIO_DIR_OUT);
  pio_direction (STEPPING_D_PORT, PIO_DIR_OUT);

  return;
}

/* ********************************************************** */
/* Tv@F@SteppingAv̎s                             */
/* ֐@F@SteppingApl_Exec                                 */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
SteppingApl_Exec (void)
{
  if (stepping_key_in != STEPPING_KEY_IN_NONE)
	{
	  /* SWꂽǂmF */
	  if (stepping_apl_is_release (stepping_key_in) == true)
		{
		  /* SWꂽ̂łƂ߂ */
		  if ((stepping_key_in == STEPPING_KEY_IN_A) ||
			  (stepping_key_in == STEPPING_KEY_IN_B))
			{
			  stepping_apl_stop ();
		  
			  stepping_timer_state = STEPPING_TIMER_STOP;
			  stepping_timer_cnt   = 0;
			}

		  stepping_key_in = STEPPING_KEY_IN_NONE;
		}
	  else
		{
		  if ((stepping_key_in == STEPPING_KEY_IN_A) ||
			  (stepping_key_in == STEPPING_KEY_IN_B))
			{
			  /* SWmF^C}[I҂ */
			  if (stepping_timer_state == STEPPING_TIMER_STOP)
				{
				  /* ^C}[X^[g */
				  stepping_apl_set_timer ();
		  
				  /* ^C}[쒆ݒ */
				  stepping_timer_state = STEPPING_TIMER_DOING;

				  /* o */
				  stepping_apl_start (stepping_apl_state);

				  /* ̏Ԃֈڍs */
				  stepping_apl_set_next_state (stepping_key_in);
				  
				}
			}
		}
	}
  else
	{
	  STEPPING_KEY_IN_TYPE input_key;

	  /* LSW͊mF */
	  if (stepping_apl_is_input (&input_key) == true)
		{
		  if ((input_key == STEPPING_KEY_IN_A) ||
			  (input_key == STEPPING_KEY_IN_B))
			{
			  /* ^C}[X^[g */
			  stepping_apl_set_timer ();
		  
			  /* ^C}[쒆ݒ */
			  stepping_timer_state = STEPPING_TIMER_DOING;

			  /* ݂̐ݒlŊJn */
			  stepping_apl_start (stepping_apl_state);

			  /* ̏Ԃֈڍs */
			  stepping_apl_set_next_state (input_key);

			}
		  else
			{
			  /* ^C}[ݒl̕ύX */
			  stepping_apl_set_timer_value (input_key);
			}

		  /* Keyۑ */
		  stepping_key_in = input_key;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@SteppingAv Timer荞                     */
/* ֐@F@SteppingApl_Irq_Timer                            */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
SteppingApl_Irq_Timer (void)
{
  if (stepping_timer_state == STEPPING_TIMER_DOING)
	{
	  if (stepping_timer_cnt <= 0)
		{
		  stepping_timer_state = STEPPING_TIMER_STOP;
		}
	  else
		{
		  stepping_timer_cnt--;
		}
	}

  return;
}


/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@LInput邩ǂ                          */
/* ֐@F@stepping_apl_is_input                            */
/* @F@STEPPING_KEY_IN_TYPE *input_key: LSWi[  */
/* ߂lF@true :  / false F Ȃ                      */
/* ********************************************************** */
static int 
stepping_apl_is_input (STEPPING_KEY_IN_TYPE *input_key)
{
  if (pio_input (PD4) == 1)
	{
	  *input_key = STEPPING_KEY_IN_A;
	  return true;
	}

  if (pio_input (PD5) == 1)
	{
	  *input_key = STEPPING_KEY_IN_B;
	  return true;
	}

  if (pio_input (PD6) == 1)
	{
	  *input_key = STEPPING_KEY_IN_C;
	  return true;
	}

  if (pio_input (PD7) == 1)
	{
	  *input_key = STEPPING_KEY_IN_D;
	  return true;
	}

  *input_key = STEPPING_KEY_IN_NONE;
  return false;
}

/* ********************************************************** */
/* Tv@F@SWꂽǂ                             */
/* ֐@F@stepping_apl_is_release                          */
/* @F@STEPPING_KEY_IN_TYPE input_key: ׂSW         */
/* ߂lF@true : ꂽ / false F Ȃ                  */
/* ********************************************************** */
static int 
stepping_apl_is_release (STEPPING_KEY_IN_TYPE input_key)
{
  unsigned char chk_key;

  switch (input_key)
	{
	case STEPPING_KEY_IN_A:
	  chk_key = PD4;
	  break;
	case STEPPING_KEY_IN_B:
	  chk_key = PD5;
	  break;
	case STEPPING_KEY_IN_C:
	  chk_key = PD6;
	  break;
	case STEPPING_KEY_IN_D:
	  chk_key = PD7;
	  break;
	default:
	  return true;
	}

  /* YSWꂽǂ`FbN */
  if (pio_input (chk_key) != 1)
	{
	  return true;
	}

  return false;
}


/* ********************************************************** */
/* Tv@F@SteppingAv o͏                          */
/* ֐@F@stepping_apl_start                               */
/* @F@STEPPING_STATE state Fo͂              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_apl_start (STEPPING_STATE state)
{
  switch (state)
	{
	case STEPPING_STATE_A:
	  pio_output (STEPPING_A_PORT, 1);
	  pio_output (STEPPING_B_PORT, 0);
	  pio_output (STEPPING_C_PORT, 0);
	  pio_output (STEPPING_D_PORT, 0);
	  break;
	case STEPPING_STATE_AB:
	  pio_output (STEPPING_A_PORT, 1);
	  pio_output (STEPPING_B_PORT, 1);
	  pio_output (STEPPING_C_PORT, 0);
	  pio_output (STEPPING_D_PORT, 0);
	  break;
	case STEPPING_STATE_B:
	  pio_output (STEPPING_A_PORT, 0);
	  pio_output (STEPPING_B_PORT, 1);
	  pio_output (STEPPING_C_PORT, 0);
	  pio_output (STEPPING_D_PORT, 0);
	  break;
	case STEPPING_STATE_BC:
	  pio_output (STEPPING_A_PORT, 0);
	  pio_output (STEPPING_B_PORT, 1);
	  pio_output (STEPPING_C_PORT, 1);
	  pio_output (STEPPING_D_PORT, 0);
	  break;
	case STEPPING_STATE_C:
	  pio_output (STEPPING_A_PORT, 0);
	  pio_output (STEPPING_B_PORT, 0);
	  pio_output (STEPPING_C_PORT, 1);
	  pio_output (STEPPING_D_PORT, 0);
	  break;
	case STEPPING_STATE_CD:
	  pio_output (STEPPING_A_PORT, 0);
	  pio_output (STEPPING_B_PORT, 0);
	  pio_output (STEPPING_C_PORT, 1);
	  pio_output (STEPPING_D_PORT, 1);
	  break;
	case STEPPING_STATE_D:
	  pio_output (STEPPING_A_PORT, 0);
	  pio_output (STEPPING_B_PORT, 0);
	  pio_output (STEPPING_C_PORT, 0);
	  pio_output (STEPPING_D_PORT, 1);
	  break;
	case STEPPING_STATE_DA:
	  pio_output (STEPPING_A_PORT, 1);
	  pio_output (STEPPING_B_PORT, 0);
	  pio_output (STEPPING_C_PORT, 0);
	  pio_output (STEPPING_D_PORT, 1);
	  break;
	default:
	  break;
	}

  return;
}


/* ********************************************************** */
/* Tv@F@SteppingAv o͒~                      */
/* ֐@F@stepping_apl_stop                                */
/* @F@Ȃ@@@@@@@@@@                         */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_apl_stop (void)
{
  pio_output (STEPPING_A_PORT, 0);
  pio_output (STEPPING_B_PORT, 0);
  pio_output (STEPPING_C_PORT, 0);
  pio_output (STEPPING_D_PORT, 0);
  return;
}

/* ********************************************************** */
/* Tv@F@SteppingAv Ԉڍs                    */
/* ֐@F@stepping_apl_set_next_state                      */
/* @F@STEPPING_KEY_IN_TYPE in_key:͂ĂSW     */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_apl_set_next_state (STEPPING_KEY_IN_TYPE in_key)
{
  switch (in_key)
	{
	case STEPPING_KEY_IN_A:
	  if (stepping_apl_state == STEPPING_STATE_DA)
		{
		  stepping_apl_state = STEPPING_STATE_A;
		}
	  else
		{
		  stepping_apl_state++;
		}
	  break;
	case STEPPING_KEY_IN_B:
	  if (stepping_apl_state == STEPPING_STATE_A)
		{
		  stepping_apl_state = STEPPING_STATE_DA;
		}
	  else
		{
		  stepping_apl_state--;
		}
	  break;
	case STEPPING_KEY_IN_C:
	case STEPPING_KEY_IN_D:
	default:
	  break;
	}
  return;
}

/* ********************************************************** */
/* Tv@F@SteppingAv ^C}[ݒ                      */
/* ֐@F@stepping_apl_set_timer                           */
/* @F@Ȃ@@@@@@@@@@                         */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_apl_set_timer (void)
{
  stepping_timer_cnt = stepping_speed_value[stepping_speed_state];
  return;
}

/* ********************************************************** */
/* Tv@F@SteppingAv ^C}[ݒlύX                */
/* ֐@F@stepping_apl_set_timer_value                     */
/* @F@STEPPING_KEY_IN_TYPE in_key:͂ĂSW     */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_apl_set_timer_value (STEPPING_KEY_IN_TYPE in_key)
{
  switch (in_key)
	{
	case STEPPING_KEY_IN_C:
	  if (stepping_speed_state != STEPPING_SPEED_HIGH)
		{
		  stepping_speed_state--;
		}
	  break;
	case STEPPING_KEY_IN_D:
	  if (stepping_speed_state != STEPPING_SPEED_LOW)
		{
		  stepping_speed_state++;
		}
	  break;
	case STEPPING_KEY_IN_A:
	case STEPPING_KEY_IN_B:
	default:
	  break;
	}

  return;
}
